@@ -15,6 +15,7 @@
 #include "BuiltinFunctions.h"
 #include "util.h"
 #include "MathStructure.h"
+#include "MathStructure-support.h"
 #include "Unit.h"
 #include "Variable.h"
 #include "Function.h"
@@ -233,19 +234,33 @@ MathStructure Calculator::convert(string str, Unit *from_unit, Unit *to_unit, co
 MathStructure Calculator::convert(const MathStructure &mstruct, KnownVariable *to_var, const EvaluationOptions &eo) {
 	if(mstruct.contains(to_var, true) > 0) return mstruct;
 	size_t n_messages = messages.size();
-	if(b_var_units && !to_var->unit().empty() && to_var->isExpression()) {
-		CompositeUnit cu("", "temporary_composite_convert", "", to_var->unit());
-		if(cu.countUnits() > 0) {
-			AliasUnit au("", "temporary_alias_convert", "", "", "", &cu, to_var->expression());
-			bool unc_rel = false;
-			if(!to_var->uncertainty(&unc_rel).empty()) au.setUncertainty(to_var->uncertainty(), unc_rel);
-			au.setApproximate(to_var->isApproximate());
-			au.setPrecision(to_var->precision());
-			MathStructure mstruct_new(convert(mstruct, &au, eo, false, false));
-			if(mstruct_new.contains(&au)) {
-				mstruct_new.replace(&au, to_var);
-				return mstruct_new;
+	if(!to_var->unit().empty() && to_var->isExpression()) {
+		int b = mstruct.containsRepresentativeOfType(STRUCT_UNIT, true, true);
+		if(b != 0 || (b < 0 && b_var_units)) {
+			beginTemporaryStopMessages();
+			CompositeUnit cu("", "temporary_composite_convert", "", to_var->unit());
+			if(!CALCULATOR->endTemporaryStopMessages() && cu.countUnits() > 0) {
+				AliasUnit au("", "temporary_alias_convert", "", "", "", &cu, to_var->expression());
+				bool unc_rel = false;
+				if(!to_var->uncertainty(&unc_rel).empty()) au.setUncertainty(to_var->uncertainty(), unc_rel);
+				au.setApproximate(to_var->isApproximate());
+				au.setPrecision(to_var->precision());
+				MathStructure mstruct_new(convert(mstruct, &au, eo, false, false));
+				if(mstruct_new.contains(&au)) {
+					mstruct_new.replace(&au, to_var);
+					if(b_var_units || !mstruct_new.containsType(STRUCT_UNIT, true)) return mstruct_new;
+				}
 			}
+		} else if(b_var_units) {
+			MathStructure mstruct_new(mstruct);
+			bool b_var_units_bak = b_var_units;
+			b_var_units = false;
+			mstruct_new /= to_var->get();
+			b_var_units = b_var_units_bak;
+			mstruct_new.eval(eo);
+			mstruct_new.multiply(to_var, true);
+			cleanMessages(mstruct, n_messages + 1);
+			return mstruct_new;
 		}
 	}
 	MathStructure mstruct_new(mstruct);
@@ -256,10 +271,12 @@ MathStructure Calculator::convert(const MathStructure &mstruct, KnownVariable *t
 	return mstruct_new;
 }
 MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo, bool always_convert, bool convert_to_mixed_units) {
-	if(!mstruct.containsType(STRUCT_UNIT, true)) return mstruct;
 	CompositeUnit *cu = NULL;
 	if(to_unit->subtype() == SUBTYPE_COMPOSITE_UNIT) cu = (CompositeUnit*) to_unit;
 	if(cu && cu->countUnits() == 0) return mstruct;
+	int exp1, exp2;
+	bool b_ratio = cu && cu->countUnits() == 2 && cu->get(1, &exp1)->baseUnit() == cu->get(2, &exp2)->baseUnit() && exp1 == -exp2;
+	if(!b_ratio && !mstruct.containsType(STRUCT_UNIT, true)) return mstruct;
 	MathStructure mstruct_new(mstruct);
 	size_t n_messages = messages.size();
 	if(to_unit->hasNonlinearRelationTo(to_unit->baseUnit()) && to_unit->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {
@@ -284,7 +301,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 		if(!b_changed && !mstruct_new.equals(mstruct, true, true)) b_changed = true;
 	}
 
-	if(!mstruct_new.isPower() && !mstruct_new.isUnit() && !mstruct_new.isMultiplication()) {
+	if(mstruct_new.size() > 0 && !mstruct_new.isPower() && !mstruct_new.isUnit() && !mstruct_new.isMultiplication()) {
 		if(mstruct_new.size() > 0) {
 			for(size_t i = 0; i < mstruct_new.size(); i++) {
 				if(aborted()) return mstruct;
@@ -309,7 +326,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 		eo2.keep_prefixes = true;
 		bool b = false;
 		if(eo.approximation == APPROXIMATION_EXACT) eo2.approximation = APPROXIMATION_TRY_EXACT;
-		if(mstruct_new.convert(to_unit, true, NULL, false, eo2, eo.keep_prefixes ? decimal_null_prefix : NULL) || always_convert) {
+		if(mstruct_new.convert(to_unit, true, NULL, false, eo2, eo.keep_prefixes ? decimal_null_prefix : NULL) || always_convert || b_ratio) {
 			b = true;
 		} else {
 			CompositeUnit *cu2 = cu;
@@ -325,6 +342,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 						break;
 					}
 					case STRUCT_MULTIPLICATION: {
+						flattenMultiplication(mstruct_new);
 						for(size_t i = 1; i <= mstruct_new.countChildren(); i++) {
 							if(aborted()) return mstruct;
 							if(mstruct_new.getChild(i)->isUnit() && cu2->containsRelativeTo(mstruct_new.getChild(i)->unit())) {
@@ -346,6 +364,15 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 				}
 			}
 		}
+		if(!b) {
+			eo2.sync_units = true;
+			eo2.keep_prefixes = false;
+			mstruct_new.eval(eo2);
+			eo2.keep_prefixes = true;
+			if(mstruct_new.convert(to_unit, true, NULL, false, eo2, eo.keep_prefixes ? decimal_null_prefix : NULL)) {
+				b = true;
+			}
+		}
 		if(b) {
 			eo2.approximation = eo.approximation;
 			eo2.sync_units = true;
@@ -360,12 +387,25 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 				if(!mbak.containsType(STRUCT_UNIT)) mstruct_new = mbak;
 			}
 
+			bool b_eval = true;
 			if(cu) {
 				MathStructure mstruct_cu(cu->generateMathStructure(false, eo.keep_prefixes));
 				Prefix *p = NULL;
 				size_t i = 1;
 				Unit *u = cu->get(i, NULL, &p);
 				while(u) {
+					size_t i2 = i + 1;
+					if(b_eval) {
+						Unit *u2 = cu->get(i2);
+						while(u2) {
+							if(u2->baseUnit() == u->baseUnit()) {
+								b_eval = false;
+								break;
+							}
+							i2++;
+							u2 = cu->get(i2);
+						}
+					}
 					mstruct_new.setPrefixForUnit(u, p);
 					i++;
 					u = cu->get(i, NULL, &p);
@@ -377,7 +417,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct, Unit *to_unit, c
 
 			eo2.sync_units = false;
 			eo2.keep_prefixes = true;
-			mstruct_new.eval(eo2);
+			if(b_eval) mstruct_new.eval(eo2);
 
 			cleanMessages(mstruct, n_messages + 1);
 
