@@ -272,6 +272,8 @@ bool Calculator::plotVectors(PlotParameters *param, const vector<MathStructure>
 		plot += "\"\n";
 	}
 
+	plot += "set termoption noenhanced\n";
+
 	switch(param->legend_placement) {
 		case PLOT_LEGEND_NONE: {plot += "set nokey\n"; break;}
 		case PLOT_LEGEND_TOP_LEFT: {plot += "set key top left\n"; break;}
@@ -319,6 +321,11 @@ bool Calculator::plotVectors(PlotParameters *param, const vector<MathStructure>
 		plot += i2s(param->x_log_base);
 		plot += "\n";
 	}
+	if(param->y_log) {
+		plot += "set logscale y ";
+		plot += i2s(param->y_log_base);
+		plot += "\n";
+	}
 	if(param->show_all_borders) {
 		plot += "set border 15\n";
 	} else {
