@@ -15,6 +15,7 @@
 #include "util.h"
 #include "Calculator.h"
 #include "MathStructure.h"
+#include "MathStructure-support.h"
 #include "Prefix.h"
 #include "BuiltinFunctions.h"
 
@@ -98,9 +99,6 @@ void Unit::setSystem(string s_system) {
 		if(ssystem == "SI" || ssystem == "si" || ssystem == "Si") {
 			b_si = true;
 			b_use_with_prefixes = true;
-		} else if(ssystem == "CGS" || ssystem == "cgs" || ssystem == "Cgs") {
-			b_si = false;
-			b_use_with_prefixes = true;
 		} else {
 			b_si = false;
 		}
@@ -769,7 +767,7 @@ bool AliasUnit::containsRelativeTo(Unit *u) const {
 bool AliasUnit::hasNonlinearRelationToBase() const {return hasNonlinearRelationTo(baseUnit()) || baseUnit()->hasNonlinearRelationToBase();}
 bool AliasUnit::hasApproximateRelationToBase(bool check_variables, bool ignore_high_precision_intervals) const {return hasApproximateRelationTo(baseUnit(), check_variables, ignore_high_precision_intervals) || baseUnit()->hasApproximateRelationToBase(check_variables, ignore_high_precision_intervals);}
 
-AliasUnit_Composite::AliasUnit_Composite(Unit *alias, int exp, Prefix *prefix_) : AliasUnit("", alias->name(), alias->plural(false), alias->singular(false), "", alias, "", exp, "") {
+AliasUnit_Composite::AliasUnit_Composite(Unit *alias, int exp, Prefix *prefix_) : AliasUnit("", alias->referenceName(), "", "", "", alias, "", exp, "") {
 	prefixv = (Prefix*) prefix_;
 }
 AliasUnit_Composite::AliasUnit_Composite(const AliasUnit_Composite *unit) {
@@ -793,10 +791,11 @@ void AliasUnit_Composite::set(const ExpressionItem *item) {
 }
 string AliasUnit_Composite::print(bool plural_, bool short_, bool use_unicode, bool (*can_display_unicode_string_function) (const char*, void*), void *can_display_unicode_string_arg) const {
 	string str = "";
+	const ExpressionName *ename = &o_unit->preferredName(short_, use_unicode, plural_, false, can_display_unicode_string_function, can_display_unicode_string_arg);
 	if(prefixv) {
-		str += prefixv->name(short_, use_unicode, can_display_unicode_string_function, can_display_unicode_string_arg);
+		str += prefixv->name(ename->abbreviation, use_unicode, can_display_unicode_string_function, can_display_unicode_string_arg);
 	}
-	str += preferredName(short_, use_unicode, plural_, false, can_display_unicode_string_function, can_display_unicode_string_arg).name;
+	str += ename->name;
 	return str;
 }
 Prefix *AliasUnit_Composite::prefix() const {
@@ -940,8 +939,10 @@ string CompositeUnit::print(bool plural_, bool short_, bool use_unicode, bool (*
 					str += "(";
 				}
 			} else {
-//				if(i > 0) str += "*";
-				if(i > 0) str += " ";
+				if(i > 0) {
+					if(use_unicode && (!can_display_unicode_string_function || (*can_display_unicode_string_function) (SIGN_MIDDLEDOT, can_display_unicode_string_arg))) str += SIGN_MIDDLEDOT;
+					else str += " ";
+				}
 			}
 			if(plural_ && i == 0 && units[i]->firstBaseExponent() > 0) {
 				str += units[i]->print(true, short_, use_unicode, can_display_unicode_string_function, can_display_unicode_string_arg);
@@ -950,13 +951,26 @@ string CompositeUnit::print(bool plural_, bool short_, bool use_unicode, bool (*
 			}
 			if(b) {
 				if(units[i]->firstBaseExponent() != -1) {
-					str += "^";
-					str += i2s(-units[i]->firstBaseExponent());
+					if(use_unicode && units[i]->firstBaseExponent() == -2 && (!can_display_unicode_string_function || (*can_display_unicode_string_function) (SIGN_POWER_2, can_display_unicode_string_arg))) str += SIGN_POWER_2;
+					else if(use_unicode && units[i]->firstBaseExponent() == -3 && (!can_display_unicode_string_function || (*can_display_unicode_string_function) (SIGN_POWER_3, can_display_unicode_string_arg))) str += SIGN_POWER_3;
+					else {
+						str += "^";
+						str += i2s(-units[i]->firstBaseExponent());
+					}
 				}
 			} else {
 				if(units[i]->firstBaseExponent() != 1) {
-					str += "^";
-					str += i2s(units[i]->firstBaseExponent());
+					if(use_unicode && units[i]->firstBaseExponent() == 2 && (!can_display_unicode_string_function || (*can_display_unicode_string_function) (SIGN_POWER_2, can_display_unicode_string_arg))) str += SIGN_POWER_2;
+					else if(use_unicode && units[i]->firstBaseExponent() == 3 && (!can_display_unicode_string_function || (*can_display_unicode_string_function) (SIGN_POWER_3, can_display_unicode_string_arg))) str += SIGN_POWER_3;
+					else {
+						str += "^";
+						if(units[i]->firstBaseExponent() < 0 && (!can_display_unicode_string_function || (*can_display_unicode_string_function) (SIGN_MINUS, can_display_unicode_string_arg))) {
+							str += SIGN_MINUS;
+							str += i2s(-units[i]->firstBaseExponent());
+						} else {
+							str += i2s(units[i]->firstBaseExponent());
+						}
+					}
 				}
 			}
 		}
@@ -1046,6 +1060,26 @@ MathStructure CompositeUnit::generateMathStructure(bool make_division, bool set_
 	}
 	return mstruct;
 }
+void remove_times_one(MathStructure &m) {
+	if(m.isMultiplication() && m.size() >= 2) {
+		for(size_t i = 0; i < m.size();) {
+			remove_times_one(m[i]);
+			if(m[i].isOne()) {
+				m.delChild(i + 1);
+				if(m.size() == 1) {
+					m.setToChild(1, true);
+					break;
+				}
+			} else {
+				i++;
+			}
+		}
+	} else {
+		for(size_t i = 0; i < m.size(); i++) {
+			remove_times_one(m[i]);
+		}
+	}
+}
 void CompositeUnit::setBaseExpression(string base_expression_) {
 	clear();
 	if(base_expression_.empty()) {
@@ -1068,35 +1102,46 @@ void CompositeUnit::setBaseExpression(string base_expression_) {
 	bool had_errors = false;
 	CALCULATOR->beginTemporaryStopMessages();
 	CALCULATOR->parse(&mstruct, base_expression_, po);
-	mstruct.eval(eo);
-	if(CALCULATOR->endTemporaryStopMessages() > 0) had_errors = true;
-	if(mstruct.isUnit()) {
-		add(mstruct.unit(), 1, mstruct.prefix());
-	} else if(mstruct.isPower() && mstruct[0].isUnit() && mstruct[1].isInteger()) {
-		add(mstruct[0].unit(), mstruct[1].number().intValue(), mstruct[0].prefix());
-	} else if(mstruct.isMultiplication()) {
-		for(size_t i = 0; i < mstruct.size(); i++) {
-			if(mstruct[i].isUnit()) {
-				add(mstruct[i].unit(), 1, mstruct[i].prefix());
-			} else if(mstruct[i].isPower() && mstruct[i][0].isUnit() && mstruct[i][1].isInteger()) {
-				add(mstruct[i][0].unit(), mstruct[i][1].number().intValue(), mstruct[i][0].prefix());
-			} else if(mstruct[i].isMultiplication()) {
-				for(size_t i2 = 0; i2 < mstruct.size(); i2++) {
-					if(mstruct[i][i2].isUnit()) {
-						add(mstruct[i][i2].unit(), 1, mstruct[i][i2].prefix());
-					} else if(mstruct[i][i2].isPower() && mstruct[i][i2][0].isUnit() && mstruct[i][i2][1].isInteger()) {
-						add(mstruct[i][i2][0].unit(), mstruct[i][i2][1].number().intValue(), mstruct[i][i2][0].prefix());
-					} else {
-						had_errors = true;
+	remove_times_one(mstruct);
+	bool b_eval = !is_unit_multiexp(mstruct);
+	while(true) {
+		if(b_eval) mstruct.eval(eo);
+		if(mstruct.isUnit()) {
+			add(mstruct.unit(), 1, mstruct.prefix());
+		} else if(mstruct.isPower() && mstruct[0].isUnit() && mstruct[1].isInteger()) {
+			add(mstruct[0].unit(), mstruct[1].number().intValue(), mstruct[0].prefix());
+		} else if(mstruct.isMultiplication()) {
+			for(size_t i = 0; i < mstruct.size(); i++) {
+				if(mstruct[i].isUnit()) {
+					add(mstruct[i].unit(), 1, mstruct[i].prefix());
+				} else if(mstruct[i].isPower() && mstruct[i][0].isUnit() && mstruct[i][1].isInteger()) {
+					add(mstruct[i][0].unit(), mstruct[i][1].number().intValue(), mstruct[i][0].prefix());
+				} else if(mstruct[i].isMultiplication()) {
+					for(size_t i2 = 0; i2 < mstruct.size(); i2++) {
+						if(mstruct[i][i2].isUnit()) {
+							add(mstruct[i][i2].unit(), 1, mstruct[i][i2].prefix());
+						} else if(mstruct[i][i2].isPower() && mstruct[i][i2][0].isUnit() && mstruct[i][i2][1].isInteger()) {
+							add(mstruct[i][i2][0].unit(), mstruct[i][i2][1].number().intValue(), mstruct[i][i2][0].prefix());
+						} else {
+							had_errors = true;
+						}
 					}
+				} else {
+					had_errors = true;
 				}
-			} else {
-				had_errors = true;
 			}
+		} else {
+			had_errors = true;
+		}
+		if(had_errors && !b_eval) {
+			had_errors = false;
+			b_eval = true;
+			clear();
+		} else {
+			break;
 		}
-	} else {
-		had_errors = true;
 	}
+	if(CALCULATOR->endTemporaryStopMessages() > 0) had_errors = true;
 	if(had_errors) CALCULATOR->error(false, _("Error(s) in unitexpression."), NULL);
 	setChanged(true);
 }
