@@ -39,11 +39,38 @@ bool is_answer_variable(Variable *v) {
 }
 
 
-string fix(string str) {
+string fix(string str, bool replace_signs = false, bool b2 = false) {
 	gsub("&", "&amp;", str);
 	gsub("<", "&lt;", str);
 	gsub(">", "&gt;", str);
 	gsub("\n", "</para><para>", str);
+	if(replace_signs) {
+		gsub(SIGN_POWER_2, "<superscript>2</superscript>", str);
+		gsub(SIGN_POWER_3, "<superscript>3</superscript>", str);
+		gsub("-", SIGN_MINUS, str);
+		size_t i = 0, i2 = 1;
+		while(true) {
+			i = str.find("^", i2);
+			i2 = i + 1;
+			if(i == string::npos || i2 == str.length()) break;
+			bool b = is_in(NUMBERS, str[i2]);
+			if(!b && (i2 + strlen(SIGN_MINUS) < str.length() && str.substr(i2, strlen(SIGN_MINUS)) == SIGN_MINUS && is_in(NUMBERS, str[i2 + strlen(SIGN_MINUS)]))) {
+				i2 += strlen(SIGN_MINUS);
+				b = true;
+			}
+			if(b) {
+				b = (i2 == str.length() - 1);
+				if(!b) b = is_in(OPERATORS COMMA SPACES PARENTHESISS, str[i2 + 1]) || (i2 + strlen(SIGN_MIDDLEDOT) + 1 < str.length() && str.substr(i2 + 1, strlen(SIGN_MIDDLEDOT)) == SIGN_MIDDLEDOT);
+				if(b) {
+					if(i2 == str.length() - 1) str += "</superscript>";
+					else str.insert(i2 + 1, "</superscript>");
+					str.erase(i, 1);
+					str.insert(i, "<superscript>");
+				}
+			}
+		}
+		gsub("*", SIGN_MULTIPLICATION, str);
+	}
 	return str;
 }
 string fixcat(string str) {
@@ -328,7 +355,7 @@ void print_function(MathFunction *f) {
 			fprintf(ffile, "<para>%s</para>\n", fix(f->description()).c_str());
 		}
 		if(!f->example(true).empty()) {
-			str = _("Example:"); str += " "; str += fix(f->example(false, ename->name));
+			str = _("Example:"); str += " "; str += fix(f->example(false, ename->name), printops.use_unicode_signs);
 			fprintf(ffile, "<para>%s</para>\n", str.c_str());
 		}
 		if(f->subtype() == SUBTYPE_DATA_SET && !((DataSet*) f)->copyright().empty()) {
@@ -371,7 +398,7 @@ void print_function(MathFunction *f) {
 			fputs("<formalpara>\n", ffile);
 			fprintf(ffile, "<title>%s</title>", _("Requirement"));
 			fputs("<para>\n", ffile);
-			fputs(fix(f->printCondition()).c_str(), ffile); fputs("\n", ffile);
+			fputs(fix(f->printCondition(), printops.use_unicode_signs).c_str(), ffile); fputs("\n", ffile);
 			fputs("</para>\n", ffile);
 			fputs("</formalpara>\n", ffile);
 		}
@@ -433,11 +460,11 @@ void print_variable(Variable *v) {
 			if(v->id() == VARIABLE_ID_PRECISION) {
 				value = _("current precision");
 			} else if(((KnownVariable*) v)->isExpression()) {
-				value = fix(CALCULATOR->localizeExpression(((KnownVariable*) v)->expression()));
+				value = fix(CALCULATOR->localizeExpression(((KnownVariable*) v)->expression()), printops.use_unicode_signs);
 				if(!((KnownVariable*) v)->uncertainty(&is_relative).empty()) {
 					if(is_relative) {value += " ("; value += _("relative uncertainty"); value += ": ";}
 					else value += SIGN_PLUSMINUS;
-					value += fix(CALCULATOR->localizeExpression(((KnownVariable*) v)->uncertainty()));
+					value += fix(CALCULATOR->localizeExpression(((KnownVariable*) v)->uncertainty()), printops.use_unicode_signs);
 					if(is_relative) {value += ")";}
 				}
 				if(((KnownVariable*) v)->expression().find_first_not_of(NUMBER_ELEMENTS EXPS) == string::npos && value.length() > 40) {
@@ -454,7 +481,7 @@ void print_variable(Variable *v) {
 				} else if(((KnownVariable*) v)->get().isVector()) {
 					value = _("vector");
 				} else {
-					value = fix(CALCULATOR->print(((KnownVariable*) v)->get(), 30));
+					value = fix(CALCULATOR->print(((KnownVariable*) v)->get(), 30), printops.use_unicode_signs);
 				}
 			}
 		} else {
@@ -523,18 +550,18 @@ void print_unit(Unit *u) {
 			}
 			case SUBTYPE_ALIAS_UNIT: {
 				AliasUnit *au = (AliasUnit*) u;
-				if(au->firstBaseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) base_unit = fix(((CompositeUnit*) au->firstBaseUnit())->print(false, true, printops.use_unicode_signs));
-				else base_unit = au->firstBaseUnit()->preferredDisplayName(printops.abbreviate_names, printops.use_unicode_signs).name;
+				base_unit = fix(au->firstBaseUnit()->print(false, printops.abbreviate_names, printops.use_unicode_signs), printops.use_unicode_signs);
 				if(au->firstBaseExponent() != 1) {
 					if(au->firstBaseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {base_unit.insert(0, 1, '('); base_unit += ")";}
-					base_unit += POWER;
+					base_unit += "<superscript>";
 					base_unit += i2s(au->firstBaseExponent());
+					base_unit += "</superscript>";
 				}
 				bool is_relative = false;
 				if(au->baseUnit() == CALCULATOR->u_euro && au->isBuiltin()) {
 					relation = "exchange rate";
 				} else {
-					relation = fix(CALCULATOR->localizeExpression(au->expression()).c_str());
+					relation = fix(CALCULATOR->localizeExpression(au->expression()).c_str(), printops.use_unicode_signs);
 					if(!au->uncertainty(&is_relative).empty()) {
 						if(is_relative) {relation += " ("; relation += _("relative uncertainty"); relation += ": ";}
 						else relation += SIGN_PLUSMINUS;
@@ -550,7 +577,7 @@ void print_unit(Unit *u) {
 				break;
 			}
 			case SUBTYPE_COMPOSITE_UNIT: {
-				base_unit = fix(((CompositeUnit*) u)->print(false, true, printops.use_unicode_signs));
+				base_unit = fix(((CompositeUnit*) u)->print(false, true, printops.use_unicode_signs), printops.use_unicode_signs);
 				relation = "";
 				break;
 			}
@@ -594,6 +621,7 @@ int main(int, char *[]) {
 		printf(_("Failed to load global definitions!\n"));
 	}
 	printops.use_unicode_signs = true;
+	printops.multiplication_sign = MULTIPLICATION_SIGN_X;
 
 	generate_functions_tree_struct();
 	generate_variables_tree_struct();
